@@ -41,9 +41,13 @@ struct rec_fex_elem_s
 {
   char *str;
 
-  rec_field_name_t field_name;
+  char *field_name;
+  char *rewrite_to;
   int max;
   int min;
+
+  char *function_name;
+  void *function_data;
 };
 
 #define REC_FEX_MAX_ELEMS 256
@@ -59,16 +63,18 @@ struct rec_fex_s
  * Static function declarations.
  */
 
-static bool rec_fex_parse_str_simple (rec_fex_t new, char *str, char *sep);
-static bool rec_fex_parse_str_subscripts (rec_fex_t new, char *str);
-static bool rec_fex_parse_elem (rec_fex_elem_t elem, char *str);
+static void rec_fex_init (rec_fex_t fex);
+
+static bool rec_fex_parse_str_simple     (rec_fex_t new,       const char *str, const char *sep);
+static bool rec_fex_parse_str_subscripts (rec_fex_t new,       const char *str);
+static bool rec_fex_parse_elem           (rec_fex_elem_t elem, const char *str);
 
 /*
  * Public functions.
  */
 
 rec_fex_t
-rec_fex_new (char *str,
+rec_fex_new (const char *str,
              enum rec_fex_kind_e kind)
 {
   rec_fex_t new;
@@ -77,7 +83,8 @@ rec_fex_new (char *str,
   new = malloc (sizeof (struct rec_fex_s));
   if (new)
     {
-      /* Initially the FEX is empty.  */
+      rec_fex_init (new);
+
       new->num_elems = 0;
       new->str = NULL;
       for (i = 0; i < REC_FEX_MAX_ELEMS; i++)
@@ -87,31 +94,35 @@ rec_fex_new (char *str,
 
       if (str != NULL)
         {
-          /* Parse the string.  */
+          /* Parse the string, using the proper parsing routine
+             depending on the kind of field expression requested by
+             the user.  */
+
           if (kind == REC_FEX_SUBSCRIPTS)
             {
               if (!rec_fex_parse_str_subscripts (new, str))
                 {
+                  /* Out of memory or parse error.  */
                   free (new);
-                  new = NULL;
+                  return NULL;
                 }
             }
           else if (kind == REC_FEX_SIMPLE)
             {
-              /* Simple FEX.  */
               if (!rec_fex_parse_str_simple (new, str, " \t\n"))
                 {
+                  /* Out of memory or parse error.  */
                   free (new);
-                  new = NULL;
+                  return NULL;
                 }
             }
           else /* REC_FEX_CSV */
             {
-              /* Simple FEX with fields separated by commas.  */
               if (!rec_fex_parse_str_simple (new, str, ","))
                 {
+                  /* Out of memory or parse error.  */
                   free (new);
-                  new = NULL;
+                  return NULL;
                 }
             }
         }
@@ -125,23 +136,91 @@ rec_fex_destroy (rec_fex_t fex)
 {
   int i;
   
-  for (i = 0; i < fex->num_elems; i++)
+  if (fex)
     {
-      if (fex->elems[i]->field_name)
+      for (i = 0; i < fex->num_elems; i++)
         {
-          rec_field_name_destroy (fex->elems[i]->field_name);
+          free (fex->elems[i]->rewrite_to);
+          free (fex->elems[i]->field_name);
+          free (fex->elems[i]->str);
+          free (fex->elems[i]);
+        }
+      
+      free (fex->str);
+      free (fex);
+    }
+}
+
+rec_fex_t
+rec_fex_dup (rec_fex_t fex)
+{
+  rec_fex_t copy = NULL;
+  size_t i = 0;
+
+  copy = malloc (sizeof (struct rec_fex_s));
+  if (copy)
+    {
+      rec_fex_init (copy);
+  
+      copy->num_elems = fex->num_elems;
+      copy->str = strdup (fex->str);
+      if (!copy->str)
+        {
+          /* Out of memory.  */
+          rec_fex_destroy (copy);
+          return NULL;
         }
 
-      free (fex->elems[i]->str);
-      free (fex->elems[i]);
+      for (i = 0; i < fex->num_elems; i++)
+        {
+          if (fex->elems[i] == NULL)
+            {
+              copy->elems[i] = NULL;
+              continue;
+            }
+
+          copy->elems[i] = malloc (sizeof (struct rec_fex_elem_s));
+          if (!copy->elems[i])
+            {
+              /* Out of memory.  */
+              rec_fex_destroy (copy);
+              return NULL;
+            }
+
+          copy->elems[i]->max = fex->elems[i]->max;
+          copy->elems[i]->min = fex->elems[i]->min;
+
+#define REC_COPY_STR_MAYBE_RETURN(FNAME)                                \
+          do                                                            \
+            {                                                           \
+               if (!fex->elems[i]->FNAME)                               \
+                 {                                                      \
+                   copy->elems[i]->FNAME = NULL;                        \
+                 }                                                      \
+               else                                                     \
+                 {                                                      \
+                   copy->elems[i]->FNAME = strdup (fex->elems[i]->FNAME); \
+                   if (!copy->elems[i]->FNAME)                          \
+                     {                                                  \
+                       /* Out of memory.  */                            \
+                       rec_fex_destroy (copy);                          \
+                       return NULL;                                     \
+                     }                                                  \
+                 }                                                      \
+            } while (0)
+
+          REC_COPY_STR_MAYBE_RETURN (str);
+          REC_COPY_STR_MAYBE_RETURN (field_name);
+          REC_COPY_STR_MAYBE_RETURN (rewrite_to);
+          REC_COPY_STR_MAYBE_RETURN (function_name);
+        }
     }
 
-  free (fex->str);
-  free (fex);
+  return copy;
 }
 
 bool
-rec_fex_check (char *str, enum rec_fex_kind_e kind)
+rec_fex_check (const char *str, enum rec_fex_kind_e kind)
 {
   char *regexp_str;
 
@@ -172,7 +251,7 @@ rec_fex_check (char *str, enum rec_fex_kind_e kind)
   return rec_match (str, regexp_str);
 }
 
-int
+size_t
 rec_fex_size (rec_fex_t fex)
 {
   return fex->num_elems;
@@ -180,7 +259,7 @@ rec_fex_size (rec_fex_t fex)
 
 rec_fex_elem_t
 rec_fex_get (rec_fex_t fex,
-             int position)
+             size_t position)
 {
   if ((position < 0) || (position >= fex->num_elems))
     {
@@ -190,29 +269,19 @@ rec_fex_get (rec_fex_t fex,
   return fex->elems[position];
 }
 
-rec_field_name_t
+const char *
 rec_fex_elem_field_name (rec_fex_elem_t elem)
 {
   return elem->field_name;
 }
 
-void
+bool
 rec_fex_elem_set_field_name (rec_fex_elem_t elem,
-                             rec_field_name_t fname)
+                             const char *fname)
 {
-  if (elem->field_name)
-    {
-      rec_field_name_destroy (elem->field_name);
-      elem->field_name = NULL;
-    }
-
-  elem->field_name = rec_field_name_dup (fname);
-}
-
-char *
-rec_fex_elem_field_name_str (rec_fex_elem_t elem)
-{
-  return elem->str;
+  free (elem->field_name);
+  elem->field_name = strdup (fname);
+  return (elem->field_name != NULL);
 }
 
 int
@@ -227,6 +296,12 @@ rec_fex_elem_max (rec_fex_elem_t elem)
   return elem->max;
 }
 
+const char *
+rec_fex_elem_rewrite_to (rec_fex_elem_t elem)
+{
+  return elem->rewrite_to;
+}
+
 void
 rec_fex_sort (rec_fex_t fex)
 {
@@ -272,13 +347,14 @@ rec_fex_str (rec_fex_t fex,
   size_t result_size;
   rec_buf_t buf;
   size_t i;
-  char *field_str;
   char *tmp;
 
   result = NULL;
   buf = rec_buf_new (&result, &result_size);
   if (buf)
     {
+      char *field_str = NULL;
+
       for (i = 0; i < fex->num_elems; i++)
         {
           if (i != 0)
@@ -293,16 +369,14 @@ rec_fex_str (rec_fex_t fex,
                 }
             }
           
-          field_str = rec_write_field_name_str (fex->elems[i]->field_name,
-                                                REC_WRITER_NORMAL);
-
-          /* People usually don't write the final ':' in fexes, so
-             remove it.  */
-          if (field_str[strlen(field_str) - 1] == ':')
+          field_str = strdup (fex->elems[i]->field_name);
+          if (!field_str)
             {
-              field_str[strlen(field_str) - 1] = '\0';
+              rec_buf_close (buf);
+              free (result);
+              return NULL;
             }
-          
+
           rec_buf_puts (field_str, buf);
           free (field_str);
 
@@ -342,7 +416,7 @@ rec_fex_str (rec_fex_t fex,
 
 bool
 rec_fex_member_p (rec_fex_t fex,
-                  rec_field_name_t fname,
+                  const char *fname,
                   int min,
                   int max)
 {
@@ -364,9 +438,9 @@ rec_fex_member_p (rec_fex_t fex,
   return res;
 }
 
-void
+rec_fex_elem_t
 rec_fex_append (rec_fex_t fex,
-                rec_field_name_t fname,
+                const char *fname,
                 int min,
                 int max)
 {
@@ -375,33 +449,90 @@ rec_fex_append (rec_fex_t fex,
   if (fex->num_elems >= REC_FEX_MAX_ELEMS)
     {
       fprintf (stderr, _("internal error: REC_FEX_MAX_ELEMS exceeded.  Please report this.\n"));
-      return;
+      return NULL;
     }
 
   new_elem = malloc (sizeof (struct rec_fex_elem_s));
   if (new_elem)
     {
-      new_elem->field_name = rec_field_name_dup (fname);
-      new_elem->str = rec_write_field_name_str (fname, REC_WRITER_NORMAL);
+      memset (new_elem, 0, sizeof (*new_elem));
+      new_elem->field_name = strdup (fname);
+      if (!new_elem->field_name)
+        {
+          /* Out of memory.  */
+          free (new_elem);
+          return NULL;
+        }
+
+      new_elem->str = strdup (fname);
+      if (!new_elem->str)
+        {
+          /* Out of memory.  */
+          free (new_elem->field_name);
+          free (new_elem);
+          return NULL;
+        }
+
       new_elem->min = min;
       new_elem->max = max;
       fex->elems[fex->num_elems++] = new_elem;
     }
+
+  return new_elem;
+}
+
+const char *
+rec_fex_elem_function_name (rec_fex_elem_t elem)
+{
+  return elem->function_name;
+}
+
+void **
+rec_fex_elem_function_data (rec_fex_elem_t elem)
+{
+  return elem->function_data;
+}
+
+bool
+rec_fex_all_calls_p (rec_fex_t fex)
+{
+  bool result = true;
+  size_t i = 0;
+
+  for (i = 0; i < fex->num_elems; i++)
+    {
+      if (fex->elems[i]->function_name == NULL)
+        {
+          result = false;
+          break;
+        }
+    }
+
+  return result;
 }
 
 /*
  * Private functions.
  */
 
+static void
+rec_fex_init (rec_fex_t fex)
+{
+  /* Initialize the field expression structure so it can be safely
+     passed to rec_fex_destroy even if its contents are not completely
+     initialized with real values.  */
+
+  memset (fex, 0 /* NULL */, sizeof (struct rec_fex_s));
+}
+
 static bool
 rec_fex_parse_str_simple (rec_fex_t new,
-                          char *str,
-                          char *sep)
+                          const char *str,
+                          const char *sep)
 {
   bool res;
   rec_fex_elem_t elem;
-  rec_field_name_t field_name;
-  char *fex_str;
+  char *fex_str, *fex_str_orig;
   char *elem_str;
   size_t i;
 
@@ -415,6 +546,7 @@ rec_fex_parse_str_simple (rec_fex_t new,
     {
       return false;
     }
+  fex_str_orig = fex_str;
 
   res = true;
 
@@ -423,11 +555,61 @@ rec_fex_parse_str_simple (rec_fex_t new,
     {
       if (strlen (elem_str) > 0)
         {
-          if ((field_name = rec_parse_field_name_str (elem_str))
-              && (elem = malloc (sizeof (struct rec_fex_elem_s))))
+          if ((elem = malloc (sizeof (struct rec_fex_elem_s))))
             {
-              /* Add an element to the FEX.  */
-              elem->field_name = field_name;
+              const char *p = elem_str;
+
+              /* Get the field name.  */
+
+              if (!rec_parse_regexp (&p,
+                                     "^" REC_FNAME_RE,
+                                     &(elem->field_name)))
+                {
+                  /* Parse error. */
+                  free (elem);
+                  res = false;
+                  break;
+                }
+
+              /* Get the subname, if any, and modify the name
+                 accordingly.  */
+
+              if (*p == '.')
+                {
+                  char *subname = NULL;
+
+                  p++;
+                  if (!rec_parse_regexp (&p,
+                                         "^" REC_FNAME_RE,
+                                         &subname))
+                    {
+                      /* Parse error.  */
+                      free (elem->field_name);
+                      free (elem);
+                      res = false;
+                      break;
+                    }
+
+                  /* Concatenate the field name and the subname.  */
+                  elem->field_name = rec_concat_strings (elem->field_name, "_", subname);
+                }
+
+              /* Check that there are no extra stuff at the end of the
+                 string.  */
+
+              if (*p != '\0')
+                {
+                  free (elem->field_name);
+                  free (elem);
+                  res = false;
+                  break;
+                }
+
+              /* Initialize other attributes of the fex entry.  */
+
+              elem->function_name = NULL;
+              elem->function_data = NULL;
+              elem->rewrite_to = NULL;
               elem->str = strdup (elem_str);
               elem->min = -1;
               elem->max = -1;
@@ -457,22 +639,24 @@ rec_fex_parse_str_simple (rec_fex_t new,
       /* Destroy parsed elements.  */
       for (i = 0; i < new->num_elems; i++)
         {
-          rec_field_name_destroy (new->elems[i]->field_name);
+          free (new->elems[i]->rewrite_to);
+          free (new->elems[i]->field_name);
           free (new->elems[i]->str);
           free (new->elems[i]);
         }
     }
 
+  free (fex_str_orig);
   return res;
 }
 
 static bool
 rec_fex_parse_str_subscripts (rec_fex_t new,
-                              char *str)
+                              const char *str)
 {
   bool res;
   char *elem_str;
-  char *fex_str;
+  char *fex_str, *fex_str_orig;
   rec_fex_elem_t elem;
   int i;
 
@@ -483,6 +667,7 @@ rec_fex_parse_str_subscripts (rec_fex_t new,
     {
       return false;
     }
+  fex_str_orig = fex_str;
 
   elem_str = strsep (&fex_str, ",");
   do
@@ -494,14 +679,18 @@ rec_fex_parse_str_subscripts (rec_fex_t new,
           res = false;
           break;
         }
+
       if (!rec_fex_parse_elem (elem, elem_str))
         {
           /* Parse error.  */
           for (i = 0; i < new->num_elems; i++)
             {
+              free (new->elems[i]->field_name);
+              free (new->elems[i]->str);
               free (new->elems[i]);
             }
 
+          free (elem);
           res = false;
           break;
         }
@@ -516,12 +705,13 @@ rec_fex_parse_str_subscripts (rec_fex_t new,
       new->str = strdup (str);
     }
 
+  free (fex_str_orig);
   return res;
 }
 
 static bool
 rec_fex_parse_elem (rec_fex_elem_t elem,
-                    char *str)
+                    const char *str)
 {
   bool ret;
   const char *p;
@@ -531,21 +721,69 @@ rec_fex_parse_elem (rec_fex_elem_t elem,
 
   /* 'Empty' part.  */
   elem->field_name = NULL;
+  elem->function_name = NULL;
+  elem->function_data = NULL;
   elem->str = NULL;
+  elem->rewrite_to = NULL;
   elem->min = -1;
   elem->max = -1;
 
+  /* The 'str' field keeps a copy of the textual entry.  */
+
+  elem->str = strdup (str);
+
+  /* Each FEX element can be either a function call or a field name
+     with an optional subscript.  */
+
+  if (rec_match (p, "^" REC_FEX_CALL))
+    {
+      /* Get the function name and the field argument and store them
+         in the FEX element.  */
+
+      if (!rec_parse_regexp (&p,
+                             "^" REC_FEX_FUNCTION_NAME,
+                             &(elem->function_name)))
+        {
+          /* Parse error.  */
+          free (elem->str);
+          return false;
+        }
+
+      p++; /* Skip the ( */
+    }
 
   /* Get the field name.  */
+      
   if (!rec_parse_regexp (&p,
                          "^" REC_FNAME_RE,
-                         &(elem->str)))
+                         &(elem->field_name)))
     {
       /* Parse error.  */
+      free (elem->str);
       return false;
     }
-  elem->field_name = rec_parse_field_name_str (elem->str);
-
+  
+  /* Get the subname and modify the name accordingly, if it
+     exists.  */
+  
+  if (*p == '.')
+    {
+      char *subname = NULL;
+      
+      p++;
+      if (!rec_parse_regexp (&p,
+                             "^" REC_FNAME_RE,
+                             &subname))
+        {
+          /* Parse error.  */
+          free (elem->str);
+          return false;
+        }
+      
+      /* Concatenate the field_name and the subname.  */
+      elem->field_name = rec_concat_strings (elem->field_name, "_", subname);
+    }
+  
   /* Get the subscripts if they are present.  */
   if (*p == '[')
     {
@@ -555,10 +793,10 @@ rec_fex_parse_elem (rec_fex_elem_t elem,
         {
           /* Parse error.  */
           free (elem->str);
-          rec_field_name_destroy (elem->field_name);
+          free (elem->field_name);
           return false;
         }
-
+      
       if (*p == '-')
         {
           p++;
@@ -567,25 +805,46 @@ rec_fex_parse_elem (rec_fex_elem_t elem,
             {
               /* Parse error.  */
               free (elem->str);
-              rec_field_name_destroy (elem->field_name);
+              free (elem->field_name);
               return false;
             }
         }
-
+      
       if (*p != ']')
         {
           /* Parse error.  */
           free (elem->str);
-          rec_field_name_destroy (elem->field_name);
+          free (elem->field_name);
           return false;
         }
       p++; /* Skip the ]  */
     }
 
+  if (elem->function_name)
+    {
+      p++; /* Skip the ) */
+    }
+      
+  /* Get the rewrite rule if it is present.  */
+  if (*p == ':')
+    {
+      p++;
+      if (!rec_parse_regexp (&p,
+                             "^" REC_FNAME_RE,
+                             &(elem->rewrite_to)))
+        {
+          /* Parse error.  */
+          free (elem->str);
+          free (elem->field_name);
+          return false;
+        }
+    }
+
   if (*p != '\0')
     {
       free (elem->str);
-      rec_field_name_destroy (elem->field_name);
+      free (elem->field_name);
+      free (elem->rewrite_to);
       return false;
     }
 
