@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2009-2014 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,7 +55,8 @@ static const char *fnames[] =
     "%type",
     "%typedef",
     "%unique",
-    "%constraint"
+    "%constraint",
+    "%allowed"
   };
 
 const char *
