@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2009-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,25 +38,30 @@ struct rec_record_s
 {
   /* Container pointer.  The semantics of this pointer depends on the
      user.  */
+
   void *container;
 
   /* type ids for the elements stored in the mset.  */
+
   int field_type;
   int comment_type;
 
   /* Localization.  */
+
   char *source;
   size_t location;
   char *location_str;
   size_t char_location;
   char *char_location_str;
 
-  /* The mset.  */
+  /* The internal multi-set storing the data.  */
+
   rec_mset_t mset;
 };
 
 /* Static functions implemented below.  */
 
+static void rec_record_init (rec_record_t record);
 static void rec_record_field_disp_fn (void *data);
 static bool rec_record_field_equal_fn (void *data1, void *data2);
 static void *rec_record_field_dup_fn (void *data);
@@ -77,6 +82,8 @@ rec_record_new (void)
 
   if (record)
     {
+      rec_record_init (record);
+
       /* The container pointer is initially NULL, until the client
          uses it for something else.  */
       record->container = NULL;
@@ -115,9 +122,9 @@ rec_record_new (void)
         }
       else
         {
-          /* Out of memory.  Report it by returning NULL.  */
+          /* Out of memory.  */
 
-          free (record);
+          rec_record_destroy (record);
           record = NULL;
         }
     }
@@ -128,11 +135,14 @@ rec_record_new (void)
 void
 rec_record_destroy (rec_record_t record)
 {
-  free (record->source);
-  free (record->location_str);
-  free (record->char_location_str);
-  rec_mset_destroy (record->mset);
-  free (record);
+  if (record)
+    {
+      free (record->source);
+      free (record->location_str);
+      free (record->char_location_str);
+      rec_mset_destroy (record->mset);
+      free (record);
+    }
 }
 
 rec_record_t
@@ -143,14 +153,26 @@ rec_record_dup (rec_record_t record)
   new = malloc (sizeof (struct rec_record_s));
   if (new)
     {
+      rec_record_init (new);
+
       new->field_type = record->field_type;
       new->comment_type = record->comment_type;
       new->mset = rec_mset_dup (record->mset);
+      if (!new->mset)
+        {
+          rec_record_destroy (new);
+          return NULL;
+        }
 
       new->source = NULL;
       if (record->source)
         {
           new->source = strdup (record->source);
+          if (!new->source)
+            {
+              rec_record_destroy (new);
+              return NULL;
+            }
         }
 
       new->location = record->location;
@@ -160,12 +182,22 @@ rec_record_dup (rec_record_t record)
       if (record->location_str)
         {
           new->location_str = strdup (record->location_str);
+          if (!new->location_str)
+            {
+              rec_record_destroy (new);
+              return NULL;
+            }
         }
 
       new->char_location_str = NULL;
       if (record->char_location_str)
         {
           new->char_location_str = strdup (record->char_location_str);
+          if (!new->char_location_str)
+            {
+              rec_record_destroy (new);
+              return NULL;
+            }
         }
 
       new->container = record->container;
@@ -277,14 +309,14 @@ rec_record_get_field_index (rec_record_t record,
 
 bool
 rec_record_field_p (rec_record_t record,
-                    rec_field_name_t field_name)
+                    const char *field_name)
 {
   return (rec_record_get_num_fields_by_name (record, field_name) > 0);
 }
 
 size_t
 rec_record_get_num_fields_by_name (rec_record_t record,
-                                   rec_field_name_t field_name)
+                                   const char *field_name)
 {
   rec_mset_iterator_t iter;
   rec_field_t field;
@@ -298,7 +330,6 @@ rec_record_get_num_fields_by_name (rec_record_t record,
           num_fields++;
         }
     }
-
   rec_mset_iterator_free (&iter);
 
   return num_fields;
@@ -306,7 +337,7 @@ rec_record_get_num_fields_by_name (rec_record_t record,
 
 rec_field_t
 rec_record_get_field_by_name (rec_record_t record,
-                              rec_field_name_t field_name,
+                              const char *field_name,
                               size_t n)
 {
   size_t num_fields = 0;
@@ -329,7 +360,6 @@ rec_record_get_field_by_name (rec_record_t record,
           num_fields++;
         }
     }
-
   rec_mset_iterator_free (&iter);
 
   return result;
@@ -337,7 +367,7 @@ rec_record_get_field_by_name (rec_record_t record,
 
 void
 rec_record_remove_field_by_name (rec_record_t record,
-                                 rec_field_name_t field_name,
+                                 const char *field_name,
                                  size_t n)
 {
   rec_field_t field;
@@ -348,9 +378,7 @@ rec_record_remove_field_by_name (rec_record_t record,
   iter = rec_mset_iterator (record->mset);
   while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void**) &field, &elem))
     {
-      if (rec_field_name_equal_p (rec_field_name (field),
-                                  field_name))
-          
+      if (rec_field_name_equal_p (rec_field_name (field), field_name))
         {
           if ((n == -1) || (n == num_fields))
             {
@@ -360,7 +388,6 @@ rec_record_remove_field_by_name (rec_record_t record,
           num_fields++;
         }
     }
-
   rec_mset_iterator_free (&iter);
 }
 
@@ -412,6 +439,7 @@ rec_record_to_comment (rec_record_t record)
           rec_buf_puts (rec_write_field_str ((rec_field_t) data,
                                              REC_WRITER_NORMAL),
                         buf);
+          rec_buf_putc ('\n', buf);
         }
       else
         {
@@ -419,6 +447,7 @@ rec_record_to_comment (rec_record_t record)
           rec_buf_puts (rec_write_comment_str (rec_comment_text ((rec_comment_t) data),
                                                REC_WRITER_NORMAL),
                         buf);
+          rec_buf_putc ('\n', buf);
         }
     }
 
@@ -546,13 +575,14 @@ rec_record_set_char_location (rec_record_t record,
 
 bool
 rec_record_contains_value (rec_record_t record,
-                           char *str,
+                           const char *str,
                            bool case_insensitive)
 {
   bool res = false;
   rec_mset_iterator_t iter;
   rec_field_t field;
-  char *field_value, *occur;
+  const char *field_value;
+  char *occur;
 
   iter = rec_mset_iterator (record->mset);
   while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void **) &field, NULL))
@@ -580,6 +610,31 @@ rec_record_contains_value (rec_record_t record,
   return res;
 }
 
+bool
+rec_record_contains_field (rec_record_t record,
+                           const char *field_name,
+                           const char *field_value)
+{
+  bool res = false;
+  rec_mset_iterator_t iter;
+  rec_field_t field;
+
+  iter = rec_mset_iterator (record->mset);
+  while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void **) &field, NULL))
+    {
+      if (rec_field_name_equal_p (field_name,
+                                  rec_field_name (field))
+          && (strcmp (field_value, rec_field_value (field)) == 0))
+        {
+          res = true;
+          break;
+        }
+    }
+  rec_mset_iterator_free (&iter);
+
+  return res;
+}
+
 void *
 rec_record_container (rec_record_t record)
 {
@@ -625,16 +680,14 @@ rec_record_uniq (rec_record_t record)
           while (rec_mset_iterator_next (&iter2, MSET_FIELD, (const void **) &field2, NULL))
             {
               if ((j != i)
-                  && rec_field_name_eql_p (rec_field_name (field1),
-                                           rec_field_name (field2))
-                  && (strcmp (rec_field_value (field1), rec_field_value (field2)) == 0))
+                  && rec_field_name_equal_p (rec_field_name (field1), rec_field_name (field2))
+                  && rec_field_name_equal_p (rec_field_value (field1), rec_field_value (field2)))
                 {
                   to_remove[j] = true;
                 }
               
               j++;
             }
-
           rec_mset_iterator_free (&iter2);
         }
 
@@ -662,10 +715,86 @@ rec_record_uniq (rec_record_t record)
   free (to_remove);
 }
 
+void
+rec_record_append (rec_record_t dest_record,
+                   rec_record_t src_record)
+{
+  rec_mset_iterator_t iter;
+  rec_field_t field;
+
+  iter = rec_mset_iterator (src_record->mset);
+  while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void **) &field, NULL))
+    {
+      if (!rec_mset_append (rec_record_mset (dest_record),
+                            MSET_FIELD,
+                            (void *) rec_field_dup (field),
+                            MSET_FIELD))
+        {
+          /* Out of memory.  Just return.  */
+          return;
+        }
+    }
+  rec_mset_iterator_free (&iter);
+}
+
+void
+rec_record_reset_marks (rec_record_t record)
+{
+  rec_mset_iterator_t iter;
+  rec_field_t field;
+
+  iter = rec_mset_iterator (record->mset);
+  while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void **) &field, NULL))
+    {
+      rec_field_set_mark (field, 0);
+    }
+  rec_mset_iterator_free (&iter);
+}
+
+bool
+rec_record_mark_field (rec_record_t record,
+                       rec_field_t field,
+                       int mark)
+{
+  rec_mset_iterator_t iter;
+  rec_field_t iter_field;
+
+  iter = rec_mset_iterator (record->mset);
+  while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void **) &iter_field, NULL))
+    {
+      if (field == iter_field)
+        {
+          rec_field_set_mark (field, mark);
+          rec_mset_iterator_free (&iter);
+          return true;
+        }
+    }
+  rec_mset_iterator_free (&iter);
+
+  return false;
+}
+
+int
+rec_record_field_mark (rec_record_t record,
+                       rec_field_t field)
+{
+  return rec_field_mark (field);
+}
+
 /*
  * Private functions
  */
 
+static void
+rec_record_init (rec_record_t record)
+{
+  /* Initialize the record structure so it can be safely passed to
+     rec_record_destroy even if its contents are not completely
+     initialized with real values.  */
+  
+  memset (record, 0 /* NULL */, sizeof (struct rec_record_s));
+}
+
 static void
 rec_record_field_disp_fn (void *data)
 {
