@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010 Jose E. Marchesi */
+/* Copyright (C) 2010-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ struct rec_sex_ast_node_s
     int integer;
     double real;
     char *string;
-    char *name;
+    char *name[2];
   } val;
 
   int index;
@@ -124,10 +124,12 @@ rec_sex_ast_node_destroy (rec_sex_ast_node_t node)
     }
   else if (node->type == REC_SEX_NAME)
     {
-      free (node->val.name);
+      free (node->val.name[0]);
+      free (node->val.name[1]);
     }
 
   free (node->fixed_val);
+  free (node);
 }
 
 enum rec_sex_ast_node_type_e
@@ -190,23 +192,36 @@ rec_sex_ast_node_set_str (rec_sex_ast_node_t node,
   node->val.string = strdup (str);
 }
 
-char *
+const char *
 rec_sex_ast_node_name (rec_sex_ast_node_t node)
 {
-  return node->val.name;
+  return node->val.name[0];
+}
+
+const char *
+rec_sex_ast_node_subname (rec_sex_ast_node_t node)
+{
+  return node->val.name[1];
 }
 
 void
 rec_sex_ast_node_set_name (rec_sex_ast_node_t node,
-                          char *name)
+                           const char *name,
+                           const char *subname)
 {
   if (node->type == REC_SEX_NAME)
     {
-      free (node->val.name);
+      free (node->val.name[0]);
+      free (node->val.name[1]);
     }
  
   node->type = REC_SEX_NAME;
-  node->val.string = strdup (name);
+  node->val.name[0] = strdup (name);
+  node->val.name[1] = NULL;
+  if (subname)
+    {
+      node->val.name[1] = strdup (subname);
+    }
 }
 
 void
@@ -250,7 +265,7 @@ rec_sex_ast_print_node (rec_sex_ast_node_t node)
     }
   if (node->type == REC_SEX_NAME)
     {
-      printf("value: %s\n", node->val.name);
+      printf("value: %s\n", node->val.name[0]);
     }
   if (node->type == REC_SEX_STR)
     {
@@ -347,5 +362,45 @@ rec_sex_ast_print (rec_sex_ast_t ast)
   rec_sex_ast_print_node (ast->top);
 }
 
+bool
+rec_sex_ast_name_p_1 (rec_sex_ast_node_t node,
+                      const char *name,
+                      size_t idx)
+{
+  size_t i = 0;
+
+  if (node)
+    {
+      if ((node->type == REC_SEX_NAME)
+          && ((node->index == -1) || (node->index < idx))
+          && (strcmp (name, node->val.name[0]) == 0))
+        {
+          return true;
+        }
+
+      for (i = 0; i < node->num_children; i++)
+        {
+          if (rec_sex_ast_name_p_1 (node->children[i], name, idx))
+            {
+              return true;
+            }
+        }
+    }
+
+  return false;
+}
+
+bool
+rec_sex_ast_name_p (rec_sex_ast_t ast,
+                    const char *name,
+                    size_t idx)
+{
+  /* Traverse the AST looking for any name node NAME[I] where I <
+     idx.  */
+
+  return rec_sex_ast_name_p_1 (ast->top,
+                               name,
+                               idx);
+}
 
 /* End of rec-sex-ast.c */
