@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.5.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,29 +43,32 @@
      REC_SEX_TOK_NAM = 261,
      REC_SEX_TOK_COLON = 262,
      REC_SEX_TOK_QM = 263,
-     REC_SEX_TOK_OR = 264,
-     REC_SEX_TOK_AND = 265,
-     REC_SEX_TOK_GT = 266,
-     REC_SEX_TOK_LT = 267,
-     REC_SEX_TOK_NEQ = 268,
-     REC_SEX_TOK_EQL = 269,
-     REC_SEX_TOK_BEFORE = 270,
-     REC_SEX_TOK_AFTER = 271,
-     REC_SEX_TOK_SAMETIME = 272,
-     REC_SEX_TOK_ADD = 273,
-     REC_SEX_TOK_SUB = 274,
-     REC_SEX_TOK_MAT = 275,
-     REC_SEX_TOK_MOD = 276,
-     REC_SEX_TOK_DIV = 277,
-     REC_SEX_TOK_MUL = 278,
-     REC_SEX_TOK_AMP = 279,
-     REC_SEX_TOK_MIN = 280,
-     REC_SEX_TOK_NEG = 281,
-     REC_SEX_TOK_NOT = 282,
-     REC_SEX_TOK_BP = 283,
-     REC_SEX_TOK_EP = 284,
-     REC_SEX_TOK_ERR = 285,
-     REC_SEX_TOK_SHARP = 286
+     REC_SEX_TOK_IMPLIES = 264,
+     REC_SEX_TOK_OR = 265,
+     REC_SEX_TOK_AND = 266,
+     REC_SEX_TOK_GTE = 267,
+     REC_SEX_TOK_LTE = 268,
+     REC_SEX_TOK_GT = 269,
+     REC_SEX_TOK_LT = 270,
+     REC_SEX_TOK_NEQ = 271,
+     REC_SEX_TOK_EQL = 272,
+     REC_SEX_TOK_BEFORE = 273,
+     REC_SEX_TOK_AFTER = 274,
+     REC_SEX_TOK_SAMETIME = 275,
+     REC_SEX_TOK_ADD = 276,
+     REC_SEX_TOK_SUB = 277,
+     REC_SEX_TOK_MAT = 278,
+     REC_SEX_TOK_MOD = 279,
+     REC_SEX_TOK_DIV = 280,
+     REC_SEX_TOK_MUL = 281,
+     REC_SEX_TOK_AMP = 282,
+     REC_SEX_TOK_MIN = 283,
+     REC_SEX_TOK_NEG = 284,
+     REC_SEX_TOK_NOT = 285,
+     REC_SEX_TOK_BP = 286,
+     REC_SEX_TOK_EP = 287,
+     REC_SEX_TOK_ERR = 288,
+     REC_SEX_TOK_SHARP = 289
    };
 #endif
 /* Tokens.  */
@@ -77,29 +78,32 @@
 #define REC_SEX_TOK_NAM 261
 #define REC_SEX_TOK_COLON 262
 #define REC_SEX_TOK_QM 263
-#define REC_SEX_TOK_OR 264
-#define REC_SEX_TOK_AND 265
-#define REC_SEX_TOK_GT 266
-#define REC_SEX_TOK_LT 267
-#define REC_SEX_TOK_NEQ 268
-#define REC_SEX_TOK_EQL 269
-#define REC_SEX_TOK_BEFORE 270
-#define REC_SEX_TOK_AFTER 271
-#define REC_SEX_TOK_SAMETIME 272
-#define REC_SEX_TOK_ADD 273
-#define REC_SEX_TOK_SUB 274
-#define REC_SEX_TOK_MAT 275
-#define REC_SEX_TOK_MOD 276
-#define REC_SEX_TOK_DIV 277
-#define REC_SEX_TOK_MUL 278
-#define REC_SEX_TOK_AMP 279
-#define REC_SEX_TOK_MIN 280
-#define REC_SEX_TOK_NEG 281
-#define REC_SEX_TOK_NOT 282
-#define REC_SEX_TOK_BP 283
-#define REC_SEX_TOK_EP 284
-#define REC_SEX_TOK_ERR 285
-#define REC_SEX_TOK_SHARP 286
+#define REC_SEX_TOK_IMPLIES 264
+#define REC_SEX_TOK_OR 265
+#define REC_SEX_TOK_AND 266
+#define REC_SEX_TOK_GTE 267
+#define REC_SEX_TOK_LTE 268
+#define REC_SEX_TOK_GT 269
+#define REC_SEX_TOK_LT 270
+#define REC_SEX_TOK_NEQ 271
+#define REC_SEX_TOK_EQL 272
+#define REC_SEX_TOK_BEFORE 273
+#define REC_SEX_TOK_AFTER 274
+#define REC_SEX_TOK_SAMETIME 275
+#define REC_SEX_TOK_ADD 276
+#define REC_SEX_TOK_SUB 277
+#define REC_SEX_TOK_MAT 278
+#define REC_SEX_TOK_MOD 279
+#define REC_SEX_TOK_DIV 280
+#define REC_SEX_TOK_MUL 281
+#define REC_SEX_TOK_AMP 282
+#define REC_SEX_TOK_MIN 283
+#define REC_SEX_TOK_NEG 284
+#define REC_SEX_TOK_NOT 285
+#define REC_SEX_TOK_BP 286
+#define REC_SEX_TOK_EP 287
+#define REC_SEX_TOK_ERR 288
+#define REC_SEX_TOK_SHARP 289
 
 
 
@@ -108,16 +112,16 @@
 typedef union YYSTYPE
 {
 
-/* Line 1676 of yacc.c  */
-#line 90 "rec-sex-tab.y"
+/* Line 2068 of yacc.c  */
+#line 92 "rec-sex-tab.y"
 
   rec_sex_ast_node_t node;
   rec_sex_ast_t ast;
 
 
 
-/* Line 1676 of yacc.c  */
-#line 121 "rec-sex-tab.h"
+/* Line 2068 of yacc.c  */
+#line 125 "rec-sex-tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
