@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2010-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -94,22 +94,25 @@ rec_sex_new (bool case_insensitive)
 void
 rec_sex_destroy (rec_sex_t sex)
 {
-  if (sex->parser)
+  if (sex)
     {
-      rec_sex_parser_destroy (sex->parser);
+      if (sex->parser)
+        {
+          rec_sex_parser_destroy (sex->parser);
+        }
+      
+      if (sex->ast)
+        {
+          rec_sex_ast_destroy (sex->ast);
+        }
+      
+      free (sex);  /* yeah! :D */
     }
-
-  if (sex->ast)
-    {
-      rec_sex_ast_destroy (sex->ast);
-    }
-
-  free (sex);  /* yeah! :D */
 }
 
 bool
 rec_sex_compile (rec_sex_t sex,
-                 char *expr)
+                 const char *expr)
 {
   bool res;
 
@@ -212,12 +215,15 @@ rec_sex_eval (rec_sex_t sex,
       goto exit;
     }
 
+  rec_record_reset_marks (record);
+
   iter = rec_mset_iterator (rec_record_mset (record));
   while (rec_mset_iterator_next (&iter, MSET_FIELD, (const void**) &field, NULL))
     {
       nf = rec_record_get_num_fields_by_name (record, rec_field_name (field));
-      if (nf > 1)
-        /* XXX.  Optimization: && (rec_sex_ast_name_p (sex->ast, field_name_str))) */
+      if ((nf > 1)
+          && (rec_record_field_mark (record, field) == 0)
+          && (rec_sex_ast_name_p (sex->ast, rec_field_name (field), nf)))
         {
           for (j = 0; j < nf; j++)
             {
@@ -229,6 +235,8 @@ rec_sex_eval (rec_sex_t sex,
                   rec_record_destroy (wrec);
                 }
 
+              rec_record_mark_field (record, wfield, 1);
+
               wrec = rec_record_dup (record);
               rec_record_remove_field_by_name (wrec,
                                                rec_field_name (field),
@@ -579,8 +587,17 @@ rec_sex_eval_node (rec_sex_t sex,
           {
             /* String comparison.  */
             res.type = REC_SEX_VAL_INT;
-            res.int_val = (strcmp (child_val1.str_val,
-                                   child_val2.str_val) == 0);
+
+            if (rec_sex_parser_case_insensitive (sex->parser))
+              {
+                res.int_val = (strcasecmp (child_val1.str_val,
+                                           child_val2.str_val) == 0);
+              }
+            else
+              {
+                res.int_val = (strcmp (child_val1.str_val,
+                                       child_val2.str_val) == 0);
+              }
           }
         else
           {
@@ -621,8 +638,17 @@ rec_sex_eval_node (rec_sex_t sex,
           {
             /* String comparison.  */
             res.type = REC_SEX_VAL_INT;
-            res.int_val = (strcmp (child_val1.str_val,
-                                   child_val2.str_val) != 0);
+
+            if (rec_sex_parser_case_insensitive (sex->parser))
+              {
+                res.int_val = (strcasecmp (child_val1.str_val,
+                                           child_val2.str_val) != 0);
+              }
+            else
+              {
+                res.int_val = (strcmp (child_val1.str_val,
+                                       child_val2.str_val) != 0);
+              }
           }
         else
           {
@@ -728,9 +754,26 @@ rec_sex_eval_node (rec_sex_t sex,
 
         res.type = REC_SEX_VAL_INT;
         res.int_val = ((diff.tv_sec == 0) && (diff.tv_nsec == 0));
+        break;
+      }
+    case REC_SEX_OP_IMPLIES:
+      {
+        int op1;
+        int op2;
+
+        GET_CHILD_VAL (child_val1, 0);
+        GET_CHILD_VAL (child_val2, 1);
+
+        ATOI_VAL (op1, child_val1);
+        ATOI_VAL (op2, child_val2);
+
+        res.type = REC_SEX_VAL_INT;
+        res.int_val = !op1 || (op1 && op2);
+
         break;
       }
     case REC_SEX_OP_LT:
+    case REC_SEX_OP_LTE:
       {
         int op1;
         int op2;
@@ -747,7 +790,15 @@ rec_sex_eval_node (rec_sex_t sex,
             ATOD_VAL (op2_real, child_val2);
 
             res.type = REC_SEX_VAL_INT;
-            res.int_val = op1_real < op2_real;
+
+            if (rec_sex_ast_node_type (node) == REC_SEX_OP_LT)
+              {
+                res.int_val = op1_real < op2_real;
+              }
+            else
+              {
+                res.int_val = op1_real <= op2_real;
+              }
           }
         else
           {
@@ -756,12 +807,21 @@ rec_sex_eval_node (rec_sex_t sex,
             ATOI_VAL (op2, child_val2);
             
             res.type = REC_SEX_VAL_INT;
-            res.int_val = op1 < op2;
+
+            if (rec_sex_ast_node_type (node) == REC_SEX_OP_LT)
+              {
+                res.int_val = op1 < op2;
+              }
+            else
+              {
+                res.int_val = op1 <= op2;
+              }
           }
 
         break;
       }
     case REC_SEX_OP_GT:
+    case REC_SEX_OP_GTE:
       {
         int op1;
         int op2;
@@ -778,7 +838,15 @@ rec_sex_eval_node (rec_sex_t sex,
             ATOD_VAL (op2_real, child_val2);
 
             res.type = REC_SEX_VAL_INT;
-            res.int_val = op1_real > op2_real;
+
+            if (rec_sex_ast_node_type (node) == REC_SEX_OP_GT)
+              {
+                res.int_val = op1_real > op2_real;
+              }
+            else
+              {
+                res.int_val = op1_real >= op2_real;
+              }
           }
         else
           {
@@ -787,7 +855,15 @@ rec_sex_eval_node (rec_sex_t sex,
             ATOI_VAL (op2, child_val2);
             
             res.type = REC_SEX_VAL_INT;
-            res.int_val = op1 > op2;
+
+            if (rec_sex_ast_node_type (node) == REC_SEX_OP_GT)
+              {
+                res.int_val = op1 > op2;
+              }
+            else
+              {
+                res.int_val = op1 >= op2;
+              }
           }
 
         break;
@@ -840,8 +916,8 @@ rec_sex_eval_node (rec_sex_t sex,
 
             res.type = REC_SEX_VAL_STR;
             res.str_val = malloc (str1_size + str2_size + 1);
-            strncpy (res.str_val, child_val1.str_val, str1_size);
-            strncpy (res.str_val + str1_size, child_val2.str_val, str2_size);
+            memcpy (res.str_val, child_val1.str_val, str1_size);
+            memcpy (res.str_val + str1_size, child_val2.str_val, str2_size);
             res.str_val[str1_size + str2_size] = '\0';
           }
         else
@@ -867,8 +943,8 @@ rec_sex_eval_node (rec_sex_t sex,
     case REC_SEX_OP_SHA:
       {
         int n;
-        rec_field_name_t field_name;
-        char *field_name_str;
+        const char *field_name    = NULL;
+        const char *field_subname = NULL;
         rec_sex_ast_node_t child;
 
         /* The child should be a Name.  */
@@ -880,10 +956,25 @@ rec_sex_eval_node (rec_sex_t sex,
             return res;
           }
 
-        field_name_str = rec_sex_ast_node_name (child);
-        field_name = rec_parse_field_name_str (field_name_str);
+        field_name = rec_sex_ast_node_name (child);
+        field_subname = rec_sex_ast_node_subname (child);
 
-        n = rec_record_get_num_fields_by_name (record, field_name);
+        if (field_subname)
+          {
+            /* Compound a field name from the name/subname pair in the
+               AST node.  */
+            
+            char *effective_name
+              = rec_concat_strings (field_name, "_", field_subname);
+
+            n = rec_record_get_num_fields_by_name (record,
+                                                   effective_name);
+            free (effective_name);
+          }
+        else
+          {
+            n = rec_record_get_num_fields_by_name (record, field_name);
+          }
 
         res.type = REC_SEX_VAL_INT;
         res.int_val = n;
@@ -935,8 +1026,8 @@ rec_sex_eval_node (rec_sex_t sex,
     case REC_SEX_NAME:
       {
         rec_field_t field;
-        rec_field_name_t field_name;
-        char *field_name_str;
+        const char *field_name;
+        const char *field_subname;
         int index;
         bool tofix;
 
@@ -947,7 +1038,8 @@ rec_sex_eval_node (rec_sex_t sex,
           }
         else
           {
-            field_name_str = rec_sex_ast_node_name (node);
+            field_name = rec_sex_ast_node_name (node);
+            field_subname = rec_sex_ast_node_subname (node);
             index = rec_sex_ast_node_index (node);
             tofix = (index != -1);
             if (index == -1)
@@ -955,13 +1047,31 @@ rec_sex_eval_node (rec_sex_t sex,
                 index = 0;
               }
 
-            field_name = rec_parse_field_name_str (field_name_str);
-            field = rec_record_get_field_by_name (record, field_name, index);
+            /* If there is a subname then the effective field name is
+               the concatenation of the name and the subname separated
+               by a '_' character.  Otherwise it is just the name.  */
+
+            {
+              if (field_subname)
+                {
+                  char *effective_field_name = malloc (sizeof (char) *
+                                                       (strlen (field_name) + strlen (field_subname) + 2));
+                  memcpy (effective_field_name, field_name, strlen(field_name));
+                  effective_field_name[strlen(field_name)] = '_';
+                  memcpy (effective_field_name + strlen(field_name) + 1, field_subname, strlen(field_subname) + 1);
+
+                  field = rec_record_get_field_by_name (record, effective_field_name, index);
+                }
+              else
+                {
+                  field = rec_record_get_field_by_name (record, field_name, index);
+                }
+            }
 
             res.type = REC_SEX_VAL_STR;
             if (field)
               {
-                res.str_val = rec_field_value (field);
+                res.str_val = strdup (rec_field_value (field));
               }
             else
               {
