@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011 Jose E. Marchesi */
+/* Copyright (C) 2010-2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <gettext.h>
 #define _(str) dgettext (PACKAGE, str)
 #include <string.h>
+#include <locale.h>
 
 #include <rec-utils.h>
 
@@ -64,7 +65,12 @@ rec_atod (const char *str,
 
   res = false;
   
+  setlocale (LC_NUMERIC, "C"); /* We want the dot to always be the
+                                  decimal separator. */
   *number = strtod (str, &end);
+  setlocale (LC_NUMERIC, ""); /* Restore the locale from the
+                                 environment.  */
+
   if ((*str != '\0') && (*end == '\0'))
     {
       /* The entire string is valid.  */
@@ -94,7 +100,7 @@ rec_extract_file (const char *str)
       /* Get the match.  */
       rec_file_length = matches.rm_eo - matches.rm_so;
       rec_file = malloc (rec_file_length + 1);
-      strncpy (rec_file, str + matches.rm_so + 1, rec_file_length - 1);
+      memcpy (rec_file, str + matches.rm_so + 1, rec_file_length - 1);
       rec_file[rec_file_length - 1] = '\0';
     }
 
@@ -122,7 +128,7 @@ rec_extract_url (const char *str)
       /* Get the match.  */
       rec_url_length = matches.rm_eo - matches.rm_so;
       rec_url = malloc (rec_url_length + 1);
-      strncpy (rec_url, str + matches.rm_so, rec_url_length);
+      memcpy (rec_url, str + matches.rm_so, rec_url_length);
       rec_url[rec_url_length] = '\0';
     }
 
@@ -138,7 +144,8 @@ rec_extract_type (const char *str)
   char *rec_type = NULL;
   size_t rec_type_length = 0;
 
-  if (regcomp (&regexp, REC_FNAME_PART_RE, REG_EXTENDED) != 0)
+  /* TODO: use a REC_TYPE_NAME_RE  */
+  if (regcomp (&regexp, REC_FNAME_RE, REG_EXTENDED) != 0)
     {
       fprintf (stderr, _("internal error: rec_int_rec_extract_url: error compiling regexp.\n"));
       return NULL;
@@ -150,7 +157,7 @@ rec_extract_type (const char *str)
       /* Get the match.  */
       rec_type_length = matches.rm_eo - matches.rm_so;
       rec_type = malloc (rec_type_length + 1);
-      strncpy (rec_type, str + matches.rm_so, rec_type_length);
+      memcpy (rec_type, str + matches.rm_so, rec_type_length);
       rec_type[rec_type_length] = '\0';
     }
 
@@ -186,7 +193,11 @@ rec_parse_int (const char **str, int *num)
       ret = false;
     }
 
-  *str = p;
+  if (ret)
+    {
+      *str = p;
+    }
+
   return ret;
 }
 
@@ -218,7 +229,7 @@ rec_parse_regexp (const char **str, const char *re, char **result)
                  since the pattern starts with a ^ rm_so shall be 0 and we
                  can use rm_eo relative to *p.  */
               *result = malloc (pm.rm_eo + 1);
-              strncpy (*result, p, pm.rm_eo);
+              memcpy (*result, p, pm.rm_eo);
               (*result)[pm.rm_eo] = '\0';
             }
 
@@ -237,7 +248,11 @@ rec_parse_regexp (const char **str, const char *re, char **result)
       regfree (&regexp);
     }
 
-  *str = p;
+  if (ret)
+    {
+      *str = p;
+    }
+
   return ret;
 }
 
@@ -412,4 +427,21 @@ rec_endian_swap (uint32_t number)
   return res;
 }
 
+char *
+rec_concat_strings (const char *str1,
+                    const char *str2,
+                    const char *str3)
+{
+  char *res = malloc (strlen (str1) + strlen (str2) + strlen (str3) + 1);
+
+  if (res)
+    {
+      memcpy (res, str1, strlen (str1));
+      memcpy (res + strlen (str1), str2, strlen (str2));
+      memcpy (res + strlen (str1) + strlen (str2), str3, strlen (str3) + 1);
+    }
+
+  return res;
+}
+
 /* End of rec-utils.c */
