@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2009, 2010, 2011, 2012, 2013 Jose E. Marchesi */
+/* Copyright (C) 2009-2014 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -240,10 +240,10 @@ rec_mset_elem_t rec_mset_search (rec_mset_t mset, void *data);
 rec_mset_iterator_t rec_mset_iterator (rec_mset_t mset);
 
 /* Advance the iterator to the next element of the given type.  The
-   data stored by the next element is stored in *DATA and a reference
-   to the element in *ELEM if ELEM is non-NULL.  The function returns
-   true if there is a next element to which iterate to, and false
-   otherwise.  */
+   data stored by the next element is stored in *DATA if DATA is
+   non-NULL and a reference to the element in *ELEM if ELEM is
+   non-NULL.  The function returns true if there is a next element to
+   which iterate to, and false otherwise.  */
 
 bool rec_mset_iterator_next (rec_mset_iterator_t *iterator,
                              rec_mset_type_t type,
@@ -285,6 +285,13 @@ void rec_mset_elem_set_data (rec_mset_elem_t elem, void *data);
 
 bool rec_mset_elem_equal_p (rec_mset_elem_t elem1, rec_mset_elem_t elem2);
 
+/* Create a copy of the data stored in a mset element and return a
+   reference to it. This uses the user-provided callback to duplicate
+   the data.  NULL is returned if there is no enough memory to
+   complete the operation.  */
+
+void *rec_mset_elem_dup_data (rec_mset_elem_t elem);
+
 /************** Sorting, grouping and other operations **************/
 
 /* Sort a given multi-set using the compare_fn callbacks provided by
@@ -396,7 +403,8 @@ enum rec_std_field_e
   REC_FIELD_TYPE,
   REC_FIELD_TYPEDEF,
   REC_FIELD_UNIQUE,
-  REC_FIELD_CONSTRAINT
+  REC_FIELD_CONSTRAINT,
+  REC_FIELD_ALLOWED
 };
 
 /******************* Field name utilities **********************/
@@ -1992,11 +2000,22 @@ void rec_writer_destroy (rec_writer_t writer);
 
 /************ Getting and setting writer properties ***************/
 
-/* Determine whether the writer must "collapse" the output records
-   when writing record sets.  */
+/* Set whether the writer must "collapse" the output records when
+   writing record sets.  */
 
 void rec_writer_set_collapse (rec_writer_t writer, bool value);
 
+/* Set whether the writer must skip comments when outputting record
+   sets and records.  */
+
+void rec_writer_set_skip_comments (rec_writer_t writer, bool value);
+
+/* Set the operation mode of the writer.  See the enumerated type
+   defined above for a list of allowed modes.  Note that the mode can
+   be changed at any time.  */
+
+void rec_writer_set_mode (rec_writer_t writer, enum rec_writer_mode_e mode);
+
 /************** Getting the properties of a writer ****************/
 
 /* Determine whether a given writer is in an EOF (end-of-file)
@@ -2014,29 +2033,28 @@ bool rec_write_string (rec_writer_t writer, const char *str);
 /* Write a comment in the given writer.  This function returns 'false'
    if there was an EOF condition.  */
 
-bool rec_write_comment (rec_writer_t writer, rec_comment_t comment, rec_writer_mode_t mode);
+bool rec_write_comment (rec_writer_t writer, rec_comment_t comment);
 
 /* Write a field name in the given writer.  This function returns
    'false' if there was an EOF condition.  */
 
-bool rec_write_field_name (rec_writer_t writer, const char *field_name, rec_writer_mode_t mode);
+bool rec_write_field_name (rec_writer_t writer, const char *field_name);
 
 /* Write a field in the given writer.  If NAME is not NULL, use it
    instead of the proper name of the field.  This function returns
    'false' if there was an EOF condition.  */
 
-bool rec_write_field (rec_writer_t writer, rec_field_t field,
-                      rec_writer_mode_t mode);
+bool rec_write_field (rec_writer_t writer, rec_field_t field);
 
 /* Write a record in the given writer.  This function returns 'false'
    if there was an EOF condition.  */
 
-bool rec_write_record (rec_writer_t writer, rec_record_t record, rec_writer_mode_t mode);
+bool rec_write_record (rec_writer_t writer, rec_record_t record);
 
 /* Write a record set to the given writer.  This function returns
    'false' if there was an EOF condition.  */
 
-bool rec_write_rset (rec_writer_t writer, rec_rset_t rset, rec_writer_mode_t mode);
+bool rec_write_rset (rec_writer_t writer, rec_rset_t rset);
 
 /* Write a database to the given writer.  This function returns
    'false' if there was an EOF condition.  */
