@@ -1,4 +1,4 @@
-/* -*- mode: C -*- Time-stamp: "2011-02-08 00:50:29 jemarch"
+/* -*- mode: C -*-
  *
  *       File:         rec-type-reg-get.c
  *       Date:         Fri Oct 29 21:30:27 2010
@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010 Jose E. Marchesi */
+/* Copyright (C) 2010, 2011 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,28 +32,26 @@
  * Test: rec_type_reg_get_nominal
  * Unit: rec_type_reg_get
  * Description:
- * + Get an existing association from a type registry.
+ * + Insert a type in a type registry and get it.
+ * + The call shall success.
  */
 START_TEST(rec_type_reg_get_nominal)
 {
   rec_type_t type;
   rec_type_t type2;
-  rec_field_name_t fname;
   rec_type_reg_t reg;
 
   reg = rec_type_reg_new ();
   fail_if (reg == NULL);
 
   /* Register a type.  */
-  type = rec_type_new ("foo int");
+  type = rec_type_new ("int");
+  rec_type_set_name (type, "foo");
   fail_if (type == NULL);
-  fname = 
-    rec_fex_elem_field_name (rec_fex_get (rec_type_descr_fex ("foo int"), 0));
-  fail_if (fname == NULL);
-  rec_type_reg_register (reg, fname, type);
+  rec_type_reg_add (reg, type);
 
   /* Get the type and compare.  */
-  type2 = rec_type_reg_get (reg, fname);
+  type2 = rec_type_reg_get (reg, "foo");
   fail_if (type2 == NULL);
   fail_if (type2 != type);
 
@@ -65,22 +63,18 @@ END_TEST
  * Test: rec_type_reg_get_nonexisting
  * Unit: rec_type_reg_get
  * Description:
- * + Try to get a nonexisting association
+ * + Try to get a nonexisting named type
  * + from a type registry.
  */
 START_TEST(rec_type_reg_get_nonexisting)
 {
-  rec_field_name_t fname;
   rec_type_reg_t reg;
 
   reg = rec_type_reg_new ();
   fail_if (reg == NULL);
-  fname = rec_fex_elem_field_name (rec_fex_get (rec_type_descr_fex ("foo int"), 0));
-  fail_if (fname == NULL);
 
-  fail_if (rec_type_reg_get (reg, fname) != NULL);
+  fail_if (rec_type_reg_get (reg, "foo") != NULL);
   
-  rec_field_name_destroy (fname);
   rec_type_reg_destroy (reg);
 }
 END_TEST
