@@ -1,4 +1,4 @@
-/* -*- mode: C -*- Time-stamp: "2010-10-29 21:38:09 jemarch"
+/* -*- mode: C -*- Time-stamp: "2011-07-16 21:18:14 jemarch"
  *
  *       File:         tsuite-rec-type-reg.c
  *       Date:         Fri Oct 29 20:58:54 2010
@@ -28,7 +28,6 @@
 
 extern TCase *test_rec_type_reg_new (void);
 extern TCase *test_rec_type_reg_destroy (void);
-extern TCase *test_rec_type_reg_register (void);
 extern TCase *test_rec_type_reg_get (void);
 
 Suite *
@@ -39,7 +38,6 @@ tsuite_rec_type_reg ()
   s = suite_create ("rec-type-reg");
   suite_add_tcase (s, test_rec_type_reg_new ());
   suite_add_tcase (s, test_rec_type_reg_destroy ());
-  suite_add_tcase (s, test_rec_type_reg_register ());
   suite_add_tcase (s, test_rec_type_reg_get ());
 
   return s;
