@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2010-2014 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,8 @@ START_TEST(rec_write_field_name_nominal)
 
   fname = "foo";
   writer = rec_writer_new_str (&str, &str_size);
-  fail_if (!rec_write_field_name (writer, fname, REC_WRITER_NORMAL));
+  rec_writer_set_mode (writer, REC_WRITER_NORMAL);
+  fail_if (!rec_write_field_name (writer, fname));
   rec_writer_destroy (writer);
   fail_if (strcmp (str, "foo:") != 0);
   free (str);
@@ -68,7 +69,8 @@ START_TEST(rec_write_field_name_sexp)
 
   fname = "foo";
   writer = rec_writer_new_str (&str, &str_size);
-  fail_if (!rec_write_field_name (writer, fname, REC_WRITER_SEXP));
+  rec_writer_set_mode (writer, REC_WRITER_SEXP);
+  fail_if (!rec_write_field_name (writer, fname));
   rec_writer_destroy (writer);
   fail_if (strcmp (str, "\"foo\"") != 0);
   free (str);
