@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010 Jose E. Marchesi */
+/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,12 +41,13 @@ START_TEST(rec_write_field_str_nominal)
   rec_field_t field;
   char *str;
 
-  field = rec_field_new (rec_parse_field_name_str ("foo"), "value");
+  field = rec_field_new ("foo", "value");
   fail_if (field == NULL);
   str = rec_write_field_str (field, REC_WRITER_NORMAL);
   fail_if (str == NULL);
   rec_field_destroy (field);
-  fail_if (strcmp (str, "foo: value\n") != 0);
+  fail_if (strcmp (str, "foo: value") != 0);
+  free (str);
 }
 END_TEST
 
@@ -61,12 +62,13 @@ START_TEST(rec_write_field_str_sexp)
   rec_field_t field;
   char *str;
 
-  field = rec_field_new (rec_parse_field_name_str ("foo"), "value");
+  field = rec_field_new ("foo", "value");
   fail_if (field == NULL);
   str = rec_write_field_str (field, REC_WRITER_SEXP);
   fail_if (str == NULL);
   rec_field_destroy (field);
-  fail_if (strcmp (str, "(field  (\"foo\") \"value\")\n") != 0);
+  fail_if (strcmp (str, "(field  \"foo\" \"value\")") != 0);
+  free (str);
 }
 END_TEST
 
