@@ -402,6 +402,22 @@ c: 30
 d: 40
 '
 
+test_declare_input_file record-with-comments \
+'foo1: v1
+# bar
+foo2: v2
+
+# jorl
+bar1: v1
+bar2: v2
+# jurl
+'
+
+test_declare_input_file ignored-first-blanks \
+'foo: v1
+bar:	v2
+'
+
 #
 # Declare tests
 #
@@ -1773,6 +1789,25 @@ test_tool recsel-sex-single-char-field-names ok \
 d: 40
 '
 
+test_tool recsel-record-with-comments ok \
+          recsel \
+          '' \
+          record-with-comments \
+'foo1: v1
+foo2: v2
+
+bar1: v1
+bar2: v2
+'
+
+test_tool recsel-ignored-first-blanks ok \
+          recsel \
+          '' \
+          ignored-first-blanks \
+'foo: v1
+bar: v2
+'
+
 #
 # Cleanup
 #
