@@ -55,11 +55,12 @@ struct csv2rec_ctx
   rec_record_t record;
 
   size_t num_fields;
+  size_t lineno;
 
-#define MAX_FIELDS 256
+#define ALLOC_FIELDS 256
   bool header_p;
   size_t num_field_names;
-  char *field_names[MAX_FIELDS];
+  char **field_names;
 };
 
 /*
@@ -194,25 +195,23 @@ void
 field_cb (void *s, size_t len, void *data)
 {
   char *str;
-  rec_field_name_t field_name;
+  char *field_name;
   rec_field_t field;
   struct csv2rec_ctx *ctx;
   size_t i;
 
   ctx = (struct csv2rec_ctx *) data;
   str = xmalloc (len + 1);
-  strncpy (str, s, len);
+  memcpy (str, s, len);
   str[len] = '\0';
 
   if (ctx->header_p)
     {
       /* Add a new field name to ctx.field_names.  */
-      ctx->num_field_names++;
-      if (ctx->num_field_names == MAX_FIELDS)
-        {
-          recutl_fatal (_("reached maximum number of fields: %d\n"),
-                        ctx->num_field_names);
-        }
+
+      if ((ctx->num_field_names % ALLOC_FIELDS) == 0)
+        ctx->field_names =
+          realloc (ctx->field_names, ((ctx->num_field_names / ALLOC_FIELDS) + 1) * (sizeof(char *) * ALLOC_FIELDS));
 
       /* Normalize the name: spaces and tabs are turned into dashes
          '_'.  */
@@ -225,15 +224,13 @@ field_cb (void *s, size_t len, void *data)
         }
 
       /* Verify that it is a valid field name.  */
-      field_name = rec_parse_field_name_str (str);
-      if (!field_name)
+      field_name = str;
+      if (!rec_field_name_p (field_name))
         {
           recutl_fatal (_("invalid field name '%s' in header\n"),
                         str);
         }
-      rec_field_name_destroy (field_name);
-
-      ctx->field_names[ctx->num_field_names] = str;
+      ctx->field_names[ctx->num_field_names++] = str;
     }
   else
     {
@@ -244,21 +241,31 @@ field_cb (void *s, size_t len, void *data)
           /* Create a new record.  */
           ctx->record = rec_record_new ();
           if (!ctx->record)
-            {
-              recutl_fatal (_("out of memory\n"));
-            }
+            recutl_out_of_memory ();
         }
       
       if (!csv2rec_omit_empty || (strlen(str) > 0))
         {
-          field_name = rec_field_name_new ();
-
           if (ctx->num_fields > ctx->num_field_names)
             {
-              recutl_fatal (_("not enough headers"));
+              char *errmsg = NULL;
+              char *source = csv2rec_csv_file;
+
+              if (!source)
+                {
+                  source = "stdin";
+                }
+
+              if (!asprintf (&errmsg,
+                             _("%s: %d: this line contains %d fields, but %d header fields were read\n"),
+                             source,
+                             ctx->lineno, ctx->num_field_names, ctx->num_fields))
+                recutl_out_of_memory ();
+
+              fprintf (stderr, errmsg);
+              exit (EXIT_FAILURE);
             }
-          rec_field_name_set (field_name, 0, ctx->field_names[ctx->num_fields]);
-          field = rec_field_new (field_name, str);
+          field = rec_field_new (ctx->field_names[ctx->num_fields], str);
           rec_mset_append (rec_record_mset (ctx->record), MSET_FIELD, (void *) field, MSET_ANY);
         }
 
@@ -272,6 +279,8 @@ record_cb (int c, void *data)
   struct csv2rec_ctx *ctx;
   ctx = (struct csv2rec_ctx *) data;
 
+  ctx->lineno++;
+
   if (ctx->header_p)
     {
       ctx->header_p = false;
@@ -283,9 +292,7 @@ record_cb (int c, void *data)
           /* Create a new record set.  */
           ctx->rset = rec_rset_new ();
           if (!ctx->rset)
-            {
-              recutl_fatal (_("out of memory\n"));
-            }
+            recutl_out_of_memory ();
 
           /* Add a type, if needed.  */
           if (csv2rec_record_type)
@@ -298,9 +305,7 @@ record_cb (int c, void *data)
             {
               ctx->db = rec_db_new ();
               if (!ctx->db)
-                {
-                  recutl_fatal (_("out of memory\n"));
-                }
+                recutl_out_of_memory ();
             }
           rec_db_insert_rset (ctx->db, ctx->rset, rec_db_size (ctx->db));
         }
@@ -329,8 +334,10 @@ process_csv (void)
   ctx.rset = NULL;
   ctx.record = NULL;
   ctx.header_p = true;
-  ctx.num_field_names = -1;
+  ctx.field_names = NULL;
+  ctx.num_field_names = 0;
   ctx.num_fields = 0;
+  ctx.lineno = 0;
 
   /* Set the files to read/write from/to.
 
