@@ -174,7 +174,7 @@ recutl_print_version (void)
 Copyright (C) %s Jose E. Marchesi.\n\
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n\
 This is free software: you are free to change and redistribute it.\n\
-There is NO WARRANTY, to the extent permitted by law.\n"), "2010, 2011, 2012, 2013");
+There is NO WARRANTY, to the extent permitted by law.\n"), "2010, 2011, 2012, 2013, 2014");
 
   puts (_("\
 \n\
