@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010, 2011, 2012 Jose E. Marchesi */
+/* Copyright (C) 2010, 2011, 2012, 2013 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,7 +70,7 @@
    {"type", required_argument, NULL, TYPE_ARG},                        \
    {"expression", required_argument, NULL, EXPRESSION_ARG},            \
    {"quick", required_argument, NULL, QUICK_ARG},                      \
-   {"num", required_argument, NULL, NUM_ARG},                          \
+   {"number", required_argument, NULL, NUM_ARG},                       \
    {"case-insensitive", no_argument, NULL, INSENSITIVE_ARG},           \
    {"random", required_argument, NULL, RANDOM_ARG}
 
@@ -82,7 +82,7 @@
     case 't':                                                  \
       {                                                        \
         recutl_type = xstrdup (optarg);                        \
-        if (!rec_field_name_part_str_p (recutl_type))          \
+        if (!rec_field_name_p (recutl_type))                   \
           {                                                    \
              recutl_fatal ("invalid record type %s\n",         \
                            recutl_type);                       \
@@ -99,10 +99,7 @@
                                                                \
         if (recutl_quick_str)                                  \
           {                                                    \
-             fprintf (stderr,                                  \
-                      "%s: cannot specify -e and also -q.\n",  \
-                      program_name);                           \
-             exit (EXIT_FAILURE);                              \
+            recutl_fatal (_("cannot specify -e and also -q\n"));\
           }                                                    \
                                                                \
          recutl_sex_str = xstrdup (optarg);                    \
@@ -113,10 +110,7 @@
             recutl_sex = rec_sex_new (recutl_insensitive);     \
             if (!rec_sex_compile (recutl_sex, recutl_sex_str)) \
              {                                                 \
-                fprintf (stderr,                               \
-                         "%s: error: invalid selection expression.\n",\
-                         program_name);                        \
-                exit (EXIT_FAILURE);                           \
+               recutl_fatal(_("invalid selection expression\n"));       \
              }                                                 \
           }                                                    \
                                                                \
@@ -128,18 +122,12 @@
                                                                \
          if (recutl_sex)                                       \
           {                                                    \
-             fprintf (stderr,                                  \
-                      "%s: cannot specify -n and also -e.\n",  \
-                      program_name);                           \
-             exit (EXIT_FAILURE);                              \
+             recutl_fatal(_("cannot specify -n and also -e\n"));\
           }                                                    \
                                                                \
         if (recutl_quick_str)                                  \
           {                                                    \
-             fprintf (stderr,                                  \
-                      "%s: cannot specify -n and also -q\n",   \
-                      program_name);                           \
-             exit (EXIT_FAILURE);                              \
+             recutl_fatal(_("cannot specify -n and also -q\n"));\
           }                                                    \
                                                                \
         if (recutl_num_indexes() != 0)                         \
@@ -159,26 +147,17 @@
       {                                                        \
         if (recutl_sex)                                        \
           {                                                    \
-             fprintf (stderr,                                  \
-                      "%s: cannot specify -m and also -e.\n",  \
-                      program_name);                           \
-             exit (EXIT_FAILURE);                              \
+             recutl_fatal (_("cannot specify -m and also -e\n"));\
           }                                                    \
                                                                \
         if (recutl_quick_str)                                  \
           {                                                    \
-             fprintf (stderr,                                  \
-                      "%s: cannot specify -m and also -q\n",   \
-                      program_name);                           \
-             exit (EXIT_FAILURE);                              \
+             recutl_fatal (_("cannot specify -m and also -q\n"));\
           }                                                    \
                                                                \
         if (recutl_num_indexes() != 0)                         \
           {                                                    \
-             fprintf (stderr,                                  \
-                      "%s: cannot specify -m and also -n\n",   \
-                      program_name);                           \
-             exit (EXIT_FAILURE);                              \
+             recutl_fatal (_("cannot specify -m and also -n\n"));\
           }                                                    \
                                                                \
         if (recutl_random)                                     \
@@ -204,14 +183,11 @@
       {                                                        \
          if (recutl_sex)                                       \
           {                                                    \
-             fprintf (stderr,                                  \
-                      "%s: cannot specify -n and also -e.\n",  \
-                      program_name);                           \
-             exit (EXIT_FAILURE);                              \
+             recutl_fatal (_("cannot specify -e and also -n\n"));\
           }                                                    \
         if (recutl_num_indexes() != 0)                         \
           {                                                    \
-            recutl_fatal (_("cannot specify -e and also -n")); \
+            recutl_fatal (_("cannot specify -e and also -n\n"));\
           }                                                    \
                                                                \
         recutl_quick_str = xstrdup (optarg);                   \
@@ -244,19 +220,16 @@ void recutl_print_help_record_selection (void);
 void recutl_error (const char *fmt, ...);
 void recutl_warning (const char *fmt, ...);
 void recutl_fatal (const char *fmt, ...);
+void recutl_out_of_memory (void);
 
 bool recutl_parse_db_from_file (FILE *in, char *file_name, rec_db_t db);
 rec_db_t recutl_build_db (int argc, char **argv);
-char *recutl_eval_field_expression (rec_fex_t fex,
-                                    rec_record_t record,
-                                    rec_writer_mode_t mode,
-                                    bool print_values_p,
-                                    bool print_in_a_row_p,
-                                    char *password);
 
 rec_db_t recutl_read_db_from_file (char *file_name);
 void recutl_write_db_to_file (rec_db_t db, char *file_name);
 
+bool recutl_file_is_writable (char *file_name);
+
 char *recutl_read_file (char *file_name);
 
 void recutl_check_integrity (rec_db_t db,
@@ -267,31 +240,27 @@ bool recutl_yesno (char *prompt);
 
 bool recutl_interactive (void);
 
+char *recutl_getpass (bool asktwice);
+
 /* Parse a list of indexes from the given string and set the internal
    recutl_indexes accordingly.  Return true if a list was found in the
    string.  Return false otherwise.  */
 
 bool recutl_index_list_parse (const char *str);
 
-/* Add a given number of random indexes to the indexes list.  The
-   generated random numbers are in the range 0..LIMIT.  */
-
-void recutl_index_add_random (size_t num, size_t limit);
-
 /* Return the number of indexes in the internal recutl_indexes data
    structure.  */
 
 size_t recutl_num_indexes (void);
 
-/* Return whether an index is stored in the internal recutl_indexes
-   data structure.  */
-
-bool recutl_index_p (size_t index);
-
 /* Reset the list of indexes to be empty.  */
 
 void recutl_reset_indexes (void);
 
+/* Return the index structure.  */
+
+size_t *recutl_index (void);
+
 #endif /* recutl.h */
 
 /* End of recutl.h */
